@@ -1,15 +1,11 @@
 use prevayler_rs::{
-    error::PrevaylerResult,
-    PrevaylerBuilder,
-    Prevayler,
-    serializer::JsonSerializer,
-    Transaction
+    error::PrevaylerResult, serializer::JsonSerializer, Prevayler, PrevaylerBuilder, Transaction,
 };
 use serde::{Deserialize, Serialize};
 
 #[derive(Serialize, Deserialize)]
 struct Increment {
-    increment: u8
+    increment: u8,
 }
 
 impl Transaction<u8> for Increment {
@@ -24,8 +20,11 @@ async fn main() -> PrevaylerResult<()> {
         .path(".")
         .serializer(JsonSerializer::new())
         .data(0 as u8)
-        .build().await?;
-    prevayler.execute_transaction(Increment{increment: 1}).await?;
+        .build()
+        .await?;
+    prevayler
+        .execute_transaction(Increment { increment: 1 })
+        .await?;
     println!("{}", prevayler.query());
     Ok(())
-}
\ No newline at end of file
+}
