@@ -1,12 +1,12 @@
 //!  # Prevayler-rs
-//! 
+//!
 //! This is a simple implementation of a system prevalance proposed by Klaus Wuestefeld in Rust.
 //! Other examples are the [prevayler](https://prevayler.org/) for Java and [prevayler-clj](https://github.com/klauswuestefeld/prevayler-clj) for Clojure.
-//! 
+//!
 //! The idea is to save in a redolog all modifications to the prevailed data. If the system restarts, it will re-apply all transactions from the redolog restoring the system state. The system may also write snapshots from time to time to speed-up the recover process.
-//! 
+//!
 //! Here is an example of a program that creates a prevailed state using an u8, increments it, print the value to the screen and closes the program.
-//! 
+//!
 //! ```rust
 //! use prevayler_rs::{
 //!     error::PrevaylerResult,
@@ -16,18 +16,18 @@
 //!     Transaction
 //! };
 //! use serde::{Deserialize, Serialize};
-//! 
+//!
 //! #[derive(Serialize, Deserialize)]
 //! struct Increment {
 //!     increment: u8
 //! }
-//! 
+//!
 //! impl Transaction<u8> for Increment {
 //!     fn execute(self, data: &mut u8) {
 //!         *data += self.increment;
 //!     }
 //! }
-//! 
+//!
 //! #[async_std::main]
 //! async fn main() -> PrevaylerResult<()> {
 //!     let mut prevayler: Prevayler<Increment, _, _> = PrevaylerBuilder::new()
@@ -40,12 +40,11 @@
 //!     Ok(())
 //! }
 //! ```
-//! 
+//!
 //! In most cases, you probably will need more than one transcation. The way that we have to do this now is to use an Enum as a main transaction that will be saved into the redolog. All transactions executed will then be converted into it.
 //!
 //! For more examples, take a look at the project tests
 
-
 pub mod error;
 mod redolog;
 pub mod serializer;
@@ -116,13 +115,6 @@ impl<T, D, S, P> PrevaylerBuilder<T, D, S, P> {
 
     /// Builds the Prevayler without snapshots. Notice that one of the Prevayler generic parameters cannot be infered by the compiler. This parameter is the type that will be serilized in the redolog.
     /// Also, if it is called without setting the path, serializer or data, this will panic.
-    /// ```rust
-    /// let mut prevayler: Prevayler<Increment, _, _> = PrevaylerBuilder::new()
-    ///     .path(".")
-    ///     .serializer(JsonSerializer::new())
-    ///     .data(0 as u8)
-    ///     .build().await?;
-    /// ```
     pub async fn build(mut self) -> PrevaylerResult<Prevayler<D, T, S>>
     where
         D: Transaction<T>,
@@ -140,7 +132,7 @@ impl<T, D, S, P> PrevaylerBuilder<T, D, S, P> {
         .await
     }
 
-    /// Similar to the [`build`](PrevaylerBuilder::build), but this will try to process any saved snapshot. Note that this method has an extra trait bound. The Serializer must know who to serialize and deserialize the prevailed data type. 
+    /// Similar to the [`build`](PrevaylerBuilder::build), but this will try to process any saved snapshot. Note that this method has an extra trait bound. The Serializer must know who to serialize and deserialize the prevailed data type.
     pub async fn build_with_snapshots(mut self) -> PrevaylerResult<Prevayler<D, T, S>>
     where
         D: Transaction<T>,
@@ -194,7 +186,7 @@ impl<D, T, S> Prevayler<D, T, S> {
 
     /// Execute the given [transaction](Transaction) and write it to the redolog.
     /// You need to have a mutable reference to the prevayler. If you are in a multithread program, you can wrapp the prevayler behind a Mutex, RwLock or anyother concurrency control system.
-    /// 
+    ///
     /// This method returns a [`PrevaylerResult`]. If it the error is returned a [`PrevaylerError::SerializationError`](crate::error::PrevaylerError::SerializationError), then you can guarantee that the prevailed state did not change. But, if it returns the error a [`PrevaylerError::IOError`](crate::error::PrevaylerError::IOError), than the data did change but the redolog is in a inconsistent state. A solution would be to force a program restart.
     pub async fn execute_transaction<TR>(&mut self, transaction: TR) -> PrevaylerResult<()>
     where
@@ -209,10 +201,25 @@ impl<D, T, S> Prevayler<D, T, S> {
         Ok(())
     }
 
+    /// Similar to the [`execute_transaction`](Prevayler::execute_transaction). But this execute [TransactionWithQuery](TransactionWithQuery) and returns its result.
+    pub async fn execute_transaction_with_query<TR, R>(
+        &mut self,
+        transaction: TR,
+    ) -> PrevaylerResult<R>
+    where
+        TR: TransactionWithQuery<T, Output = R> + Into<D>,
+        S: Serializer<D>,
+    {
+        let result = transaction.execute_and_return(&mut self.data);
+        let transaction: D = transaction.into();
+        let serialized = self.redo_log.serialize(&transaction)?;
+        self.redo_log.write_to_log(serialized).await?;
+        Ok(result)
+    }
 
     /// Like [execute_transaction](Prevayler::execute_transaction), it executes the give transaction and write it to the redolog. But, if a transaction panics for some reason, this gurantees that the prevailed state will not be changed.
     /// This gurantee comes wiht a cost. The entire prevailed state is cloned everytime that a transaction is executed.
-    /// 
+    ///
     /// Think twice before using this method. Your transactions should probably not be able to panic. Try to do any code that can fail before the transaction and only do the state change inside it.
     pub async fn execute_transaction_panic_safe<TR>(
         &mut self,
@@ -248,22 +255,38 @@ impl<D, T, S> Prevayler<D, T, S> {
     }
 }
 
-
 /// The trait that defines the transaction behaivour.
 /// You must implement it to all your transactions.
-/// 
+///
 /// Notice that the execute method does not return a Result. You should handle your errors and gurantee that the prevailed state is in a consistent state.
-/// 
+///
 /// Transactions should be deterministic. Do all non-deterministic code outside the transaction and let your transaction just update the values.
 pub trait Transaction<T> {
     fn execute(self, data: &mut T);
 }
 
+/// This trait is similar to the [Transaction](Transaction). But, it also returns a value. All the same rules of the Transaction trait must be true to a TrascationWithQuery.
+///
+/// A blanket implementations is done for the [Transaction](Transaction) trait for everyone that implements TransactionWithQuery.
+pub trait TransactionWithQuery<T> {
+    type Output;
+    fn execute_and_return(&self, data: &mut T) -> Self::Output;
+}
+
+impl<T, D> Transaction<T> for D
+where
+    D: TransactionWithQuery<T>,
+{
+    fn execute(self, data: &mut T) {
+        self.execute_and_return(data);
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{
         error::PrevaylerResult, serializer::JsonSerializer, Prevayler, PrevaylerBuilder,
-        Transaction,
+        Transaction, TransactionWithQuery,
     };
     use async_std::sync::Mutex;
     use serde::{Deserialize, Serialize};
@@ -291,6 +314,11 @@ mod tests {
         value: u8,
     }
 
+    #[derive(Serialize, Deserialize)]
+    struct AddToSecondElement {
+        value: u8,
+    }
+
     impl Transaction<(u8, u8)> for ChangeFirstElement {
         fn execute(self, data: &mut (u8, u8)) {
             data.0 = self.value;
@@ -316,12 +344,22 @@ mod tests {
         }
     }
 
+    impl TransactionWithQuery<(u8, u8)> for AddToSecondElement {
+        type Output = u8;
+        fn execute_and_return(&self, data: &mut (u8, u8)) -> u8 {
+            let old_value = data.0;
+            data.0 += self.value;
+            return old_value;
+        }
+    }
+
     #[derive(Serialize, Deserialize)]
     enum Transactions {
         ChangeFirstElement(ChangeFirstElement),
         ChangeSecondElement(ChangeSecondElement),
         AddToFirstElement(AddToFirstElement),
         AddToSecondElementFailling(AddToSecondElementFailling),
+        AddToSecondElement(AddToSecondElement),
     }
 
     impl Transaction<(u8, u8)> for Transactions {
@@ -339,6 +377,9 @@ mod tests {
                 Transactions::AddToSecondElementFailling(e) => {
                     e.execute(data);
                 }
+                Transactions::AddToSecondElement(e) => {
+                    e.execute(data);
+                }
             };
         }
     }
@@ -367,6 +408,12 @@ mod tests {
         }
     }
 
+    impl Into<Transactions> for AddToSecondElement {
+        fn into(self) -> Transactions {
+            Transactions::AddToSecondElement(self)
+        }
+    }
+
     #[async_std::test]
     async fn test_transaction() -> PrevaylerResult<()> {
         let temp = TempDir::default();
@@ -561,4 +608,32 @@ mod tests {
         }
         Ok(())
     }
+
+    #[async_std::test]
+    async fn test_transaction_with_query() -> PrevaylerResult<()> {
+        let temp = TempDir::default();
+        let data = (3, 4);
+        let mut prevayler: Prevayler<Transactions, _, _> = PrevaylerBuilder::new()
+            .path(&temp.as_os_str())
+            .max_log_size(10)
+            .serializer(JsonSerializer::new())
+            .data(data)
+            .build()
+            .await?;
+
+        assert_eq!(
+            3,
+            prevayler
+                .execute_transaction_with_query(AddToSecondElement { value: 7 })
+                .await?
+        );
+        assert_eq!(
+            10,
+            prevayler
+                .execute_transaction_with_query(AddToSecondElement { value: 5 })
+                .await?
+        );
+        assert_eq!(&(15, 4), prevayler.query());
+        Ok(())
+    }
 }
